@@ -112,16 +112,6 @@ typedef enum {
 	E_EWS_BODY_TYPE_TEXT
 } EEwsBodyType;
 
-typedef struct {
-	gchar *as_url;
-	gchar *oab_url;
-
-	/* all the below variables are for future use */
-	gchar *oof_url;
-	gpointer future1;
-	gpointer future2;
-} EwsUrls;
-
 typedef struct {
 	gchar *id;
 	gchar *dn;
@@ -129,6 +119,7 @@ typedef struct {
 } EwsOAL;
 
 typedef struct {
+	gchar *type;
 	guint32 seq;
 	guint32 ver;
 	guint32 size;
@@ -180,18 +171,18 @@ typedef struct {
 	gchar *prop_name;
 	gchar *prop_id;
 	gchar *prop_type;
-} EwsExtendedFieldURI;
+} EEwsExtendedFieldURI;
 
 typedef struct {
 	gchar *field_uri;
 	gchar *field_index;
-} EwsIndexedFieldURI;
+} EEwsIndexedFieldURI;
 
 typedef struct {
 	gchar *field_uri;
 	GSList *extended_furis;
 	GSList *indexed_furis;
-} EwsAdditionalProps;
+} EEwsAdditionalProps;
 
 typedef struct {
 	gchar *order;
@@ -199,12 +190,211 @@ typedef struct {
 	gpointer field_uri;
 } EwsSortOrder;
 
+typedef struct {
+	gchar *id;
+	gsize len;
+} EwsPhotoAttachmentInfo;
+
+typedef enum {
+	E_EWS_NOTIFICATION_EVENT_COPIED = 0,
+	E_EWS_NOTIFICATION_EVENT_CREATED,
+	E_EWS_NOTIFICATION_EVENT_DELETED,
+	E_EWS_NOTIFICATION_EVENT_MODIFIED,
+	E_EWS_NOTIFICATION_EVENT_MOVED,
+	E_EWS_NOTIFICATION_EVENT_STATUS
+} EEwsNotificationEventType;
+
+typedef struct {
+	EEwsNotificationEventType type;
+	gboolean is_item;
+	gchar *folder_id;
+	gchar *old_folder_id;
+} EEwsNotificationEvent;
+
+/*
+ * <To Kind=""/>
+ */
+typedef struct {
+	gchar *kind;
+	gchar *value;
+} EEwsCalendarTo;
+
+/*
+ * <AbsoluteDateTransition>
+ *     <To/>
+ *     <DateTime/>
+ * </AbsoluteDateTransition>
+ */
+typedef struct {
+	EEwsCalendarTo *to;
+	gchar *date_time;
+} EEwsCalendarAbsoluteDateTransition;
+
+/*
+ * <RecurringDayTransition>
+ *     <To/>
+ *     <TimeOffset/>
+ *     <Month/>
+ *     <DayOfWeek/>
+ *     <Ocurrence/>
+ * </RecurringDayTransition>
+ */
+typedef struct {
+	EEwsCalendarTo *to;
+	gchar *time_offset;
+	gchar *month;
+	gchar *day_of_week;
+	gchar *occurrence;
+} EEwsCalendarRecurringDayTransition;
+
+/*
+ * <RecurringDateTransition>
+ *     <To/>
+ *     <TimeOffset/>
+ *     <Month/>
+ *     <Day/>
+ * </RecurringDateTransition>
+ */
+typedef struct {
+	EEwsCalendarTo *to;
+	gchar *time_offset;
+	gchar *month;
+	gchar *day;
+} EEwsCalendarRecurringDateTransition;
+
+/*
+ * <Period Bias="" Name="" Id=""/>
+ */
+typedef struct {
+	gchar *bias;
+	gchar *name;
+	gchar *id;
+} EEwsCalendarPeriod;
+
+/*
+ * <TransitionsGroup Id="">
+ *     <Transition>
+ *         <To/>
+ *     <Transition>
+ *     <AbsoluteDateTransition/>
+ *     <RecurringDayTransition/>
+ *     <RecurringDateTransition/>
+ * </TransitionsGroup>
+ */
+typedef struct {
+	gchar *id;
+	EEwsCalendarTo *transition;
+	GSList *absolute_date_transitions; /* EEwsCalendarAbsoluteDateTransition */
+	GSList *recurring_day_transitions; /* EEwsCalendarRecurringDayTransition */
+	GSList *recurring_date_transitions; /* EEwsCalendarRecurringDateTransition */
+} EEwsCalendarTransitionsGroup;
+
+/*
+ * <Transitions Id="">
+ *     <Transition>
+ *         <To/>
+ *     <Transition>
+ *     <AbsoluteDateTransition/>
+ *     <RecurringDayTransition/>
+ *     <RecurringDateTransition/>
+ * </Transitions>
+ */
+typedef struct {
+	EEwsCalendarTo *transition;
+	GSList *absolute_date_transitions; /* EEwsCalendarAbsoluteDateTransition */
+	GSList *recurring_day_transitions; /* EEwsCalendarRecurringDayTransition */
+	GSList *recurring_date_transitions; /* EEwsCalendarRecurringDateTransition */
+} EEwsCalendarTransitions;
+
+/*
+ * <TimeZoneDefinition Id="" Name="">
+ *     <Periods>
+ *         <Period/>
+ *     </Periods>
+ *     <TransitionsGroups>
+ *         <TransitionsGroup/>
+ *     </TransitionsGroups>
+ *     <Transitions/>
+ * </TimeZoneDefinition>
+ */
+typedef struct {
+	gchar *name;
+	gchar *id;
+	GSList *periods; /* EEwsCalendarPeriod */
+	GSList *transitions_groups; /* EEwsCalendarTrasitionsGroup */
+	EEwsCalendarTransitions *transitions;
+} EEwsCalendarTimeZoneDefinition;
+
+EEwsCalendarTo *
+		e_ews_calendar_to_new		(void);
+void		e_ews_calendar_to_free		(EEwsCalendarTo *to);
+
+EEwsCalendarAbsoluteDateTransition *
+		e_ews_calendar_absolute_date_transition_new
+						(void);
+void		e_ews_calendar_absolute_date_transition_free
+						(EEwsCalendarAbsoluteDateTransition *adt);
+
+EEwsCalendarRecurringDayTransition *
+		e_ews_calendar_recurring_day_transition_new
+						(void);
+void		e_ews_calendar_recurring_day_transition_free
+						(EEwsCalendarRecurringDayTransition *rdayt);
+
+EEwsCalendarRecurringDateTransition *
+		e_ews_calendar_recurring_date_transition_new
+						(void);
+void		e_ews_calendar_recurring_date_transition_free
+						(EEwsCalendarRecurringDateTransition *rdatet);
+
+EEwsCalendarPeriod *
+		e_ews_calendar_period_new	(void);
+void		e_ews_calendar_period_free	(EEwsCalendarPeriod *period);
+
+EEwsCalendarTransitionsGroup *
+		e_ews_calendar_transitions_group_new
+						(void);
+void		e_ews_calendar_transitions_group_free
+						(EEwsCalendarTransitionsGroup *tg);
+
+EEwsCalendarTransitions *
+		e_ews_calendar_transitions_new	(void);
+void		e_ews_calendar_transitions_free	(EEwsCalendarTransitions *transitions);
+
+EEwsCalendarTimeZoneDefinition *
+		e_ews_calendar_time_zone_definition_new
+						(void);
+void		e_ews_calendar_time_zone_definition_free
+						(EEwsCalendarTimeZoneDefinition *tzd);
+
+EEwsExtendedFieldURI *
+		e_ews_extended_field_uri_new	(void);
+void		e_ews_extended_field_uri_free	(EEwsExtendedFieldURI *ex_field_uri);
+
+EEwsIndexedFieldURI *
+		e_ews_indexed_field_uri_new	(void);
+void		e_ews_indexed_field_uri_free	(EEwsIndexedFieldURI *id_field_uri);
+
+EEwsAdditionalProps *
+		e_ews_additional_props_new	(void);
+void		e_ews_additional_props_free	(EEwsAdditionalProps *add_props);
+
+EEwsNotificationEvent *
+		e_ews_notification_event_new	(void);
+void		e_ews_notification_event_free	(EEwsNotificationEvent *event);
+
 void		ews_oal_free			(EwsOAL *oal);
 void		ews_oal_details_free		(EwsOALDetails *details);
 
+void		e_ews_connection_utils_unref_in_thread
+						(gpointer object);
+
 GType		e_ews_connection_get_type	(void);
 EEwsConnection *e_ews_connection_new		(const gchar *uri,
 						 CamelEwsSettings *settings);
+EEwsConnection *e_ews_connection_new_full	(const gchar *uri,
+						 CamelEwsSettings *settings,
+						 gboolean allow_connection_reuse);
 const gchar *	e_ews_connection_get_uri	(EEwsConnection *cnc);
 const gchar *	e_ews_connection_get_password	(EEwsConnection *cnc);
 gchar *		e_ews_connection_dup_password	(EEwsConnection *cnc);
@@ -212,6 +402,12 @@ void		e_ews_connection_set_password	(EEwsConnection *cnc,
 						 const gchar *password);
 const gchar *	e_ews_connection_get_impersonate_user
 						(EEwsConnection *cnc);
+GProxyResolver *
+		e_ews_connection_ref_proxy_resolver
+						(EEwsConnection *cnc);
+void		e_ews_connection_set_proxy_resolver
+						(EEwsConnection *cnc,
+						 GProxyResolver *proxy_resolver);
 CamelEwsSettings *
 		e_ews_connection_ref_settings	(EEwsConnection *cnc);
 SoupSession *	e_ews_connection_ref_soup_session
@@ -253,7 +449,7 @@ void		e_ews_connection_sync_folder_items
 						 const gchar *old_sync_state,
 						 const gchar *fid,
 						 const gchar *default_props,
-						 const gchar *additional_props,
+						 const EEwsAdditionalProps *add_props,
 						 guint max_entries,
 						 GCancellable *cancellable,
 						 GAsyncReadyCallback callback,
@@ -273,7 +469,7 @@ gboolean	e_ews_connection_sync_folder_items_sync
 						 const gchar *old_sync_state,
 						 const gchar *fid,
 						 const gchar *default_props,
-						 const gchar *additional_props,
+						 const EEwsAdditionalProps *add_props,
 						 guint max_entries,
 						 gchar **new_sync_state,
 						 gboolean *includes_last_item,
@@ -292,7 +488,7 @@ void		e_ews_connection_find_folder_items
 						 gint pri,
 						 EwsFolderId *fid,
 						 const gchar *props,
-						 EwsAdditionalProps *add_props,
+						 const EEwsAdditionalProps *add_props,
 						 EwsSortOrder *sort_order,
 						 const gchar *query,
 						 EEwsFolderType type,
@@ -311,7 +507,7 @@ gboolean	e_ews_connection_find_folder_items_sync
 						 gint pri,
 						 EwsFolderId *fid,
 						 const gchar *default_props,
-						 EwsAdditionalProps *add_props,
+						 const EEwsAdditionalProps *add_props,
 						 EwsSortOrder *sort_order,
 						 const gchar *query,
 						 EEwsFolderType type,
@@ -321,11 +517,24 @@ gboolean	e_ews_connection_find_folder_items_sync
 						 GCancellable *cancellable,
 						 GError **error);
 
+EEwsServerVersion
+		e_ews_connection_get_server_version
+						(EEwsConnection *cnc);
+void		e_ews_connection_set_server_version
+						(EEwsConnection *cnc,
+						 EEwsServerVersion version);
+void		e_ews_connection_set_server_version_from_string
+						(EEwsConnection *cnc,
+						 const gchar *version);
+gboolean	e_ews_connection_satisfies_server_version
+						(EEwsConnection *cnc,
+						 EEwsServerVersion versio);
+
 void		e_ews_connection_get_items	(EEwsConnection *cnc,
 						 gint pri,
 						 const GSList *ids,
 						 const gchar *default_props,
-						 const gchar *additional_props,
+						 const EEwsAdditionalProps *add_props,
 						 gboolean include_mime,
 						 const gchar *mime_directory,
 						 EEwsBodyType body_type,
@@ -343,7 +552,7 @@ gboolean	e_ews_connection_get_items_sync	(EEwsConnection *cnc,
 						 gint pri,
 						 const GSList *ids,
 						 const gchar *default_props,
-						 const gchar *additional_props,
+						 const EEwsAdditionalProps *add_props,
 						 gboolean include_mime,
 						 const gchar *mime_directory,
 						 EEwsBodyType body_type,
@@ -355,7 +564,7 @@ gboolean	e_ews_connection_get_items_sync	(EEwsConnection *cnc,
 
 void		e_ews_connection_delete_items	(EEwsConnection *cnc,
 						 gint pri,
-						 GSList *ids,
+						 const GSList *ids,
 						 EwsDeleteType delete_type,
 						 EwsSendMeetingCancellationsType send_cancels,
 						 EwsAffectedTaskOccurrencesType affected_tasks,
@@ -369,7 +578,7 @@ gboolean	e_ews_connection_delete_items_finish
 gboolean	e_ews_connection_delete_items_sync
 						(EEwsConnection *cnc,
 						 gint pri,
-						 GSList *ids,
+						 const GSList *ids,
 						 EwsDeleteType delete_type,
 						 EwsSendMeetingCancellationsType send_cancels,
 						 EwsAffectedTaskOccurrencesType affected_tasks,
@@ -585,6 +794,29 @@ gboolean	e_ews_connection_delete_folder_sync
 						 GCancellable *cancellable,
 						 GError **error);
 
+void		e_ews_connection_empty_folder	(EEwsConnection *cnc,
+						 gint pri,
+						 const gchar *folder_id,
+						 gboolean is_distinguished_id,
+						 const gchar *delete_type,
+						 gboolean delete_subfolders,
+						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
+						 gpointer user_data);
+gboolean	e_ews_connection_empty_folder_finish
+						(EEwsConnection *cnc,
+						 GAsyncResult *result,
+						 GError **error);
+gboolean	e_ews_connection_empty_folder_sync
+						(EEwsConnection *cnc,
+						 gint pri,
+						 const gchar *folder_id,
+						 gboolean is_distinguished_id,
+						 const gchar *delete_type,
+						 gboolean delete_subfolders,
+						 GCancellable *cancellable,
+						 GError **error);
+
 void		e_ews_connection_update_folder	(EEwsConnection *cnc,
 						 gint pri,
 						 EEwsRequestCreationCallback create_cb,
@@ -626,7 +858,7 @@ gboolean	e_ews_connection_move_folder_sync
 void		e_ews_connection_get_folder	(EEwsConnection *cnc,
 						 gint pri,
 						 const gchar *folder_shape,
-						 EwsAdditionalProps *add_props,
+						 const EEwsAdditionalProps *add_props,
 						 GSList *folder_ids,
 						 GCancellable *cancellable,
 						 GAsyncReadyCallback callback,
@@ -640,7 +872,7 @@ gboolean	e_ews_connection_get_folder_sync
 						(EEwsConnection *cnc,
 						 gint pri,
 						 const gchar *folder_shape,
-						 EwsAdditionalProps *add_props,
+						 const EEwsAdditionalProps *add_props,
 						 GSList *folder_ids,
 						 GSList **folders,
 						 GCancellable *cancellable,
@@ -650,7 +882,7 @@ void		e_ews_connection_move_items	(EEwsConnection *cnc,
 						 gint pri,
 						 const gchar *folder_id,
 						 gboolean docopy,
-						 GSList *ids,
+						 const GSList *ids,
 						 GCancellable *cancellable,
 						 GAsyncReadyCallback callback,
 						 gpointer user_data);
@@ -664,7 +896,7 @@ gboolean	e_ews_connection_move_items_sync
 						 gint pri,
 						 const gchar *folder_id,
 						 gboolean docopy,
-						 GSList *ids,
+						 const GSList *ids,
 						 GSList **items_ret,
 						 GCancellable *cancellable,
 						 GError **error);
@@ -674,38 +906,44 @@ void		e_ews_connection_create_attachments
 						 gint pri,
 						 const EwsId *parent,
 						 const GSList *files,
+						 gboolean is_contact_photo,
 						 GCancellable *cancellable,
 						 GAsyncReadyCallback callback,
 						 gpointer user_data);
-GSList *	e_ews_connection_create_attachments_finish
+gboolean	e_ews_connection_create_attachments_finish
 						(EEwsConnection *cnc,
 						 gchar **change_key,
+						 GSList **attachments_ids,
 						 GAsyncResult *result,
 						 GError **error);
-GSList *	e_ews_connection_create_attachments_sync
+gboolean	e_ews_connection_create_attachments_sync
 						(EEwsConnection *cnc,
 						 gint pri,
 						 const EwsId *parent,
 						 const GSList *files,
+						 gboolean is_contact_photo,
 						 gchar **change_key,
+						 GSList **attachments_ids,
 						 GCancellable *cancellable,
 						 GError **error);
 
 void		e_ews_connection_delete_attachments
 						(EEwsConnection *cnc,
 						 gint pri,
-						 const GSList *ids,
+						 const GSList *attachments_ids,
 						 GCancellable *cancellable,
 						 GAsyncReadyCallback callback,
 						 gpointer user_data);
-GSList *	e_ews_connection_delete_attachments_finish
+gboolean	e_ews_connection_delete_attachments_finish
 						(EEwsConnection *cnc,
 						 GAsyncResult *result,
+						 GSList **parents_ids,
 						 GError **error);
-GSList *	e_ews_connection_delete_attachments_sync
+gboolean	e_ews_connection_delete_attachments_sync
 						(EEwsConnection *cnc,
 						 gint pri,
-						 const GSList *ids,
+						 const GSList *attachments_ids,
+						 GSList **parents_ids,
 						 GCancellable *cancellable,
 						 GError **error);
 
@@ -721,12 +959,12 @@ void		e_ews_connection_get_attachments
 						 GCancellable *cancellable,
 						 GAsyncReadyCallback callback,
 						 gpointer user_data);
-GSList *	e_ews_connection_get_attachments_finish
+gboolean	e_ews_connection_get_attachments_finish
 						(EEwsConnection *cnc,
 						 GAsyncResult *result,
 						 GSList **items,
 						 GError **error);
-GSList *	e_ews_connection_get_attachments_sync
+gboolean	e_ews_connection_get_attachments_sync
 						(EEwsConnection *cnc,
 						 gint pri,
 						 const gchar *comp_uid,
@@ -757,12 +995,15 @@ gboolean	e_ews_connection_get_oal_detail_sync
 						(EEwsConnection *cnc,
 						 const gchar *oal_id,
 						 const gchar *oal_element,
+						 const gchar *old_etag,
 						 GSList **elements,
+						 gchar **etag,
 						 GCancellable *cancellable,
 						 GError **error);
 void		e_ews_connection_get_oal_detail	(EEwsConnection *cnc,
 						 const gchar *oal_id,
 						 const gchar *oal_element,
+						 const gchar *etag,
 						 GCancellable *cancellable,
 						 GAsyncReadyCallback callback,
 						 gpointer user_data);
@@ -770,6 +1011,7 @@ gboolean	e_ews_connection_get_oal_detail_finish
 						(EEwsConnection *cnc,
 						 GAsyncResult *result,
 						 GSList **elements,
+						 gchar **etag,
 						 GError **error);
 
 void		e_ews_connection_get_free_busy	(EEwsConnection *cnc,
@@ -996,6 +1238,49 @@ gboolean	e_ews_connection_find_folder_sync
 						 GSList **folders,
 						 GCancellable *cancellable,
 						 GError **error);
+void		e_ews_connection_query_auth_methods
+						(EEwsConnection *cnc,
+						 gint pri,
+						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
+						 gpointer user_data);
+gboolean	e_ews_connection_query_auth_methods_finish
+						(EEwsConnection *cnc,
+						 GAsyncResult *result,
+						 GSList **auth_methods,
+						 GError **error);
+gboolean	e_ews_connection_query_auth_methods_sync
+						(EEwsConnection *cnc,
+						 gint pri,
+						 GSList **auth_methods,
+						 GCancellable *cancellable,
+						 GError **error);
+void		e_ews_connection_enable_notifications_sync
+						(EEwsConnection *cnc,
+						 GSList *folders,
+						 guint *subscription_key);
+void		e_ews_connection_disable_notifications_sync
+						(EEwsConnection *cnc,
+						 guint subscription_key);
+void		e_ews_connection_get_server_time_zones
+						(EEwsConnection *cnc,
+						 gint pri,
+						 GSList *msdn_locations,
+						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
+						 gpointer user_data);
+gboolean	e_ews_connection_get_server_time_zones_finish
+						(EEwsConnection *cnc,
+						 GAsyncResult *result,
+						 GSList **tzds, /* EEwsCalendarTimeZoneDefinition */
+						 GError **error);
+gboolean	e_ews_connection_get_server_time_zones_sync
+						(EEwsConnection *cnc,
+						 gint pri,
+						 GSList *msdn_locations,
+						 GSList **tzds, /* EEwsCalendarTimeZoneDefinition */
+						 GCancellable *cancellable,
+						 GError **error);
 
 G_END_DECLS
 
